from mlflow.tracking.context.abstract_context import RunContextProvider


class PluginRunContextProvider(RunContextProvider):
    """RunContextProvider provided through plugin system"""

    def in_context(self):
        return False

    def tags(self):
        return {"test": "tag"}



import hashlib
import json
import yaml
import os
import sys
import shutil
from six.moves import urllib
import subprocess
import tempfile
import logging
import posixpath
import docker
import platform
from os.path import expanduser
from os.path import sep as separator
import requests
from requests.exceptions import HTTPError
import boto3
from urllib.parse import urlparse

from mlflow.projects.backend.abstract_backend import AbstractBackend
import mlflow.projects.databricks
from mlflow.projects import _get_or_create_conda_env, _get_conda_command, _run_entry_point,\
        _get_entry_point_command
import mlflow.tracking as tracking
from mlflow.entities import RunStatus
from mlflow.exceptions import ExecutionException, MlflowException
from mlflow.projects.submitted_run import LocalSubmittedRun, SubmittedRun
from mlflow.projects.utils import (
    _get_storage_dir, fetch_and_validate_project, get_or_create_run, load_project,
    _MLFLOW_LOCAL_BACKEND_RUN_ID_CONFIG
)
from mlflow.projects.backend import loader
from mlflow.tracking.context.git_context import _get_git_commit
from mlflow.tracking.fluent import _get_experiment_id
from mlflow.store.artifact.artifact_repository_registry import get_artifact_repository
from mlflow.store.artifact.azure_blob_artifact_repo import AzureBlobArtifactRepository
from mlflow.store.artifact.gcs_artifact_repo import GCSArtifactRepository
from mlflow.store.artifact.hdfs_artifact_repo import HdfsArtifactRepository
from mlflow.store.artifact.local_artifact_repo import LocalArtifactRepository
from mlflow.store.artifact.s3_artifact_repo import S3ArtifactRepository
from mlflow.utils import databricks_utils, file_utils, process
from mlflow.utils.file_utils import path_to_local_sqlite_uri, path_to_local_file_uri
from mlflow.utils.mlflow_tags import (
    MLFLOW_PROJECT_ENV, MLFLOW_DOCKER_IMAGE_URI, MLFLOW_DOCKER_IMAGE_ID, MLFLOW_PROJECT_BACKEND,
)
from mlflow.utils.uri import get_db_profile_from_uri, is_databricks_uri

from infinstor_mlflow_plugin.cognito_auth_rest_store import CognitoAuthenticatedRestStore

class InfinStorSubmittedRun(SubmittedRun):
    """
    A run that just does nothing
    """

    def __init__(self, run_id):
        self._run_id = run_id

    def wait(self):
        return True

    def get_status(self):
        return RunStatus.FINISHED

    def cancel(self):
        pass

    @property
    def run_id(self):
        return self._run_id



def upload_objects(bucket_name, path_in_bucket, local_path):
    if (path_in_bucket[0] == '/'):
        path_in_bucket = path_in_bucket[1:]
    print('upload_objects: Entered. bucket=' + bucket_name + ', path_in_bucket=' + path_in_bucket\
            + ', local_path=' + local_path)
    # s3_resource = boto3.resource("s3", region_name="us-east-1")
    s3_resource = boto3.resource("s3")

    try:
        my_bucket = s3_resource.Bucket(bucket_name)

        for path, subdirs, files in os.walk(local_path):
            path = path.replace("\\","/")
            directory_name = path.replace(local_path, "")
            for onefile in files:
                src_path = os.path.join(path, onefile)
                dst_path = path_in_bucket + directory_name + '/' + onefile
                my_bucket.upload_file(src_path, dst_path)
    except Exception as err:
        print(err)

class PluginInfinStorProjectBackend(AbstractBackend):
    def run(self, project_uri, entry_point, params,
            version, backend_config, experiment_id, tracking_store_uri):

        print("PluginInfinStorProjectBackend: project_uri=" + str(project_uri)\
                + ", entry_point=" + str(entry_point)\
                + ", params=" + str(params)\
                + ", version=" + str(version)\
                + ", backend_config=" + str(backend_config)\
                + ", experiment_id=" + str(experiment_id)\
                + ", tracking_store_uri=" + str(tracking_store_uri))

        work_dir = fetch_and_validate_project(project_uri, version, entry_point, params)
        active_run = get_or_create_run(None, project_uri, experiment_id, work_dir, version,
                                       entry_point, params)

        print('active_run=' + str(active_run))
        print('active_run.info=' + str(active_run.info))

        artifact_uri = active_run.info.artifact_uri
        run_id = active_run.info.run_id

        tags = active_run.data.tags
        if (tags['mlflow.source.type'] != 'PROJECT'):
            raise ValueError('mlflow.source_type must be PROJECT. Instead it is '\
                    + tags['mlflow.source.type'])

        pdst = urlparse(artifact_uri)
        bucket_name = pdst.netloc
        if (pdst.path[0] == '/'):
            path_in_bucket = pdst.path[1:]
        else:
            path_in_bucket = pdst.path
        upload_objects(bucket_name, path_in_bucket + '/project_files', tags['mlflow.source.name'])

        project = load_project(work_dir)
        tracking.MlflowClient().set_tag(active_run.info.run_id, MLFLOW_PROJECT_BACKEND, "infinstor")

        body = dict()
        body['project_files_bucket'] = bucket_name
        body['project_files_path_in_bucket'] = path_in_bucket
        body['params'] = params
        body['run_id'] = run_id
        body['experiment_id'] = str(experiment_id)
        home = expanduser("~")
        tokfile = expanduser("~") + separator + '.infinstor' + separator + '/token'
        data = open(tokfile, "r").read()
        body['dot_infinstor_contents'] = data

        cog = CognitoAuthenticatedRestStore()
        headers = {
                'Content-Type': 'application/x-amz-json-1.1',
                'Authorization' : 'Bearer ' + cog.get_token_string()
                }
        url = 'https://mlflow.' + cog.get_service() + '.com/api/2.0/mlflow/projects/run-project'

        try:
            response = requests.post(url, data=json.dumps(body), headers=headers)
            response.raise_for_status()
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')
            raise
        except Exception as err:
            print(f'Other error occurred: {err}')
            raise
        else:
            return InfinStorSubmittedRun(active_run.info.run_id)




from mlflow.store.tracking.rest_store import RestStore
from mlflow.utils.rest_utils import MlflowHostCreds
from infinstor_mlflow_plugin.tokenfile import get_token
from os.path import expanduser
from os.path import sep as separator

class CognitoAuthenticatedRestStore(RestStore):
    def cognito_host_creds(self):
        tokfile = expanduser("~") + separator + '.infinstor' + separator + '/token'
        token, service = get_token(tokfile, False)
        return MlflowHostCreds('https://mlflow.' + service + '.com:443/', token=token)

    def get_service(self):
        tokfile = expanduser("~") + separator + '.infinstor' + separator + '/token'
        token, service = get_token(tokfile, False)
        return service

    def get_token_string(self):
        tokfile = expanduser("~") + separator + '.infinstor' + separator + '/token'
        token, service = get_token(tokfile, False)
        return token

    def __init__(self, store_uri=None, artifact_uri=None):
        super().__init__(self.cognito_host_creds)




from six.moves import urllib

from mlflow.store.model_registry.sqlalchemy_store import SqlAlchemyStore


class PluginRegistrySqlAlchemyStore(SqlAlchemyStore):
    def __init__(self, store_uri=None):
        path = urllib.parse.urlparse(store_uri).path if store_uri else None
        self.is_plugin = True
        super(PluginRegistrySqlAlchemyStore, self).__init__(path)



import os
from mlflow.deployments import BaseDeploymentClient

f_deployment_name = 'fake_deployment_name'


class PluginDeploymentClient(BaseDeploymentClient):
    def create_deployment(self, name, model_uri, flavor=None, config=None):
        if config and config.get('raiseError') == 'True':
            raise RuntimeError("Error requested")
        return {'name': f_deployment_name, 'flavor': flavor}

    def delete_deployment(self, name):
        return None

    def update_deployment(self, name, model_uri=None, flavor=None, config=None):
        return {'flavor': flavor}

    def list_deployments(self):
        if os.environ.get('raiseError') == 'True':
            raise RuntimeError('Error requested')
        return [f_deployment_name]

    def get_deployment(self, name):
        return {'key1': 'val1', 'key2': 'val2'}

    def predict(self, deployment_name, df):
        return 1


def run_local(name, model_uri, flavor=None, config=None):
    print(f"Deployed locally at the key {name} using the model from {model_uri}. "
          f"It's flavor is {flavor} and config is {config}")


def target_help():
    return "Target help is called"



from mlflow.store.artifact.local_artifact_repo import LocalArtifactRepository


class PluginLocalArtifactRepository(LocalArtifactRepository):
    """LocalArtifactRepository provided through plugin system"""
    is_plugin = True



#!/usr/bin/env python
import sys
import getpass
import json
import builtins
from . import servicedefs
from infinstor_mlflow_plugin.tokenfile import read_token_file, write_token_file
from requests.exceptions import HTTPError
import requests
from os.path import expanduser
from os.path import sep as separator
import datetime

def get_creds():
    if sys.stdin.isatty():
        print("Enter your InfinStor service username and password")
        username = input("Username: ")
        password = getpass.getpass("Password: ")
    else:
        username = sys.stdin.readline().rstrip()
        password = sys.stdin.readline().rstrip()
    return username, password

def login_and_update_token_file(username, password):
    postdata = dict()
    auth_parameters = dict()
    auth_parameters['USERNAME'] = username
    auth_parameters['PASSWORD'] = password
    postdata['AuthParameters'] = auth_parameters
    postdata['AuthFlow'] = "USER_PASSWORD_AUTH"
    postdata['ClientId'] = builtins.clientid

    payload = json.dumps(postdata)

    url = 'https://cognito-idp.us-east-1.amazonaws.com:443/'
    headers = {
            'Content-Type': 'application/x-amz-json-1.1',
            'X-Amz-Target' : 'AWSCognitoIdentityProviderService.InitiateAuth'
            }

    try:
        response = requests.post(url, data=payload, headers=headers)
        response.raise_for_status()
    except HTTPError as http_err:
        print(f'HTTP error occurred: {http_err}')
        raise
    except Exception as err:
        print(f'Other error occurred: {err}')
        raise
    else:
        authres = response.json()['AuthenticationResult']
        token = authres['IdToken']
        refresh_token = authres['RefreshToken']
        token_time = round(datetime.datetime.timestamp(datetime.datetime.utcnow()))
        tokfile = expanduser("~") + separator + '.infinstor' + separator + '/token'
        write_token_file(tokfile, token_time, token, refresh_token, builtins.clientid,\
                builtins.service)
        return True

def main():
    username, password = get_creds()
    return login_and_update_token_file(username, password)

if __name__ == "__main__":
    if (main()):
        exit(0)
    else:
        exit(255)
